@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,34 +18,49 @@
 
 #include "config.h"
 #include "task.h"
-#include <lua.h>
-#include <event.h>
+#include "lua/lua_common.h"
+#include "contrib/libev/ev.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * @file stat_api.h
  * High level statistics API
  */
 
-#define RSPAMD_STAT_TOKEN_FLAG_TEXT (1 << 0)
-#define RSPAMD_STAT_TOKEN_FLAG_META (1 << 1)
-#define RSPAMD_STAT_TOKEN_FLAG_LUA_META (1 << 2)
-#define RSPAMD_STAT_TOKEN_FLAG_EXCEPTION (1 << 3)
-#define RSPAMD_STAT_TOKEN_FLAG_SUBJECT (1 << 4)
-#define RSPAMD_STAT_TOKEN_FLAG_UNIGRAM (1 << 5)
+#define RSPAMD_STAT_TOKEN_FLAG_TEXT (1u << 0)
+#define RSPAMD_STAT_TOKEN_FLAG_META (1u << 1)
+#define RSPAMD_STAT_TOKEN_FLAG_LUA_META (1u << 2)
+#define RSPAMD_STAT_TOKEN_FLAG_EXCEPTION (1u << 3)
+#define RSPAMD_STAT_TOKEN_FLAG_HEADER (1u << 4)
+#define RSPAMD_STAT_TOKEN_FLAG_UNIGRAM (1u << 5)
+#define RSPAMD_STAT_TOKEN_FLAG_UTF (1u << 6)
+#define RSPAMD_STAT_TOKEN_FLAG_NORMALISED (1u << 7)
+#define RSPAMD_STAT_TOKEN_FLAG_STEMMED (1u << 8)
+#define RSPAMD_STAT_TOKEN_FLAG_BROKEN_UNICODE (1u << 9)
+#define RSPAMD_STAT_TOKEN_FLAG_STOP_WORD (1u << 10)
+#define RSPAMD_STAT_TOKEN_FLAG_SKIPPED (1u << 11)
+#define RSPAMD_STAT_TOKEN_FLAG_INVISIBLE_SPACES (1u << 12)
+#define RSPAMD_STAT_TOKEN_FLAG_EMOJI (1u << 13)
 
 typedef struct rspamd_stat_token_s {
-	const gchar *begin;
-	gsize len;
+	rspamd_ftok_t original;        /* utf8 raw */
+	rspamd_ftok_unicode_t unicode; /* array of unicode characters, normalized, lowercased */
+	rspamd_ftok_t normalized;      /* normalized and lowercased utf8 */
+	rspamd_ftok_t stemmed;         /* stemmed utf8 */
 	guint flags;
 } rspamd_stat_token_t;
 
+#define RSPAMD_TOKEN_VALUE_TYPE float
 typedef struct token_node_s {
 	guint64 data;
 	guint window_idx;
 	guint flags;
 	rspamd_stat_token_t *t1;
 	rspamd_stat_token_t *t2;
-	gdouble values[];
+	RSPAMD_TOKEN_VALUE_TYPE values[0];
 } rspamd_token_t;
 
 struct rspamd_stat_ctx;
@@ -66,20 +81,20 @@ typedef enum rspamd_stat_result_e {
  * Initialise statistics modules
  * @param cfg
  */
-void rspamd_stat_init (struct rspamd_config *cfg, struct event_base *ev_base);
+void rspamd_stat_init(struct rspamd_config *cfg, struct ev_loop *ev_base);
 
 /**
  * Finalize statistics
  */
-void rspamd_stat_close (void);
+void rspamd_stat_close(void);
 
 /**
  * Tokenize task
  * @param st_ctx
  * @param task
  */
-void rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
-							  struct rspamd_task *task);
+void rspamd_stat_process_tokenize(struct rspamd_stat_ctx *st_ctx,
+								  struct rspamd_task *task);
 
 /**
  * Classify the task specified and insert symbols if needed
@@ -88,8 +103,8 @@ void rspamd_stat_process_tokenize (struct rspamd_stat_ctx *st_ctx,
  * @param err error returned
  * @return TRUE if task has been classified
  */
-rspamd_stat_result_t rspamd_stat_classify (struct rspamd_task *task,
-		lua_State *L, guint stage, GError **err);
+rspamd_stat_result_t rspamd_stat_classify(struct rspamd_task *task,
+										  lua_State *L, guint stage, GError **err);
 
 
 /**
@@ -97,7 +112,7 @@ rspamd_stat_result_t rspamd_stat_classify (struct rspamd_task *task,
  * @param task
  * @return
  */
-gboolean rspamd_stat_check_autolearn (struct rspamd_task *task);
+gboolean rspamd_stat_check_autolearn(struct rspamd_task *task);
 
 /**
  * Learn task as spam or ham, task must be processed prior to this call
@@ -108,10 +123,10 @@ gboolean rspamd_stat_check_autolearn (struct rspamd_task *task);
  * @param err error returned
  * @return TRUE if task has been learned
  */
-rspamd_stat_result_t rspamd_stat_learn (struct rspamd_task *task,
-		gboolean spam, lua_State *L, const gchar *classifier,
-		guint stage,
-		GError **err);
+rspamd_stat_result_t rspamd_stat_learn(struct rspamd_task *task,
+									   gboolean spam, lua_State *L, const gchar *classifier,
+									   guint stage,
+									   GError **err);
 
 /**
  * Get the overall statistics for all statfile backends
@@ -119,11 +134,15 @@ rspamd_stat_result_t rspamd_stat_learn (struct rspamd_task *task,
  * @param total_learns the total number of learns is stored here
  * @return array of statistical information
  */
-rspamd_stat_result_t rspamd_stat_statistics (struct rspamd_task *task,
-		struct rspamd_config *cfg,
-		guint64 *total_learns,
-		ucl_object_t **res);
+rspamd_stat_result_t rspamd_stat_statistics(struct rspamd_task *task,
+											struct rspamd_config *cfg,
+											guint64 *total_learns,
+											ucl_object_t **res);
 
-void rspamd_stat_unload (void);
+void rspamd_stat_unload(void);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* STAT_API_H_ */
